"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import astar

#FOR CELL MAPS
StraightCost = 10
DiagonalCost = 14

running = False
current = None

def _diagonalTo(a, b):
  return b[0] == a[0] - 1 and (b[1] == a[1] - 1 or b[1] == a[1] + 1) or b[0] == a[0] + 1 and (b[1] == a[1] - 1 or b[1] == a[1] + 1)

def g_cellbased(a, b):
    if _diagonalTo(a,b):
        return DiagonalCost
    else:
        return StraightCost
    
g_function = g_cellbased

def h_diagonal(pos, end):
    h_diagonal = min(abs(pos[0]-end[0]), abs(pos[1]-end[1]))
    h_straight = (abs(pos[0]-end[0]) + abs(pos[1]-end[1]))
    return DiagonalCost * h_diagonal + StraightCost * (h_straight - 2*h_diagonal)
    
def h_manhattan(pos, end):
    return StraightCost * (abs(pos[0] - end[0]) + abs(pos[1] - end[1]))

h_function = h_manhattan

def get_neighbors(pos):
    l = [ (pos[0]-1, pos[1]),
          (pos[0]+1, pos[1]),
          (pos[0], pos[1]-1),
          (pos[0], pos[1]+1),
          (pos[0]+1,pos[1] +1), 
          (pos[0]-1,pos[1] +1),
          (pos[0]-1,pos[1] -1),
          (pos[0]+1,pos[1] -1)]
    
    i=0
    while i < len(l):
        todel = False
        if  l[i] not in astar.moveable:
            del l[i]
            i-= 1
        i+= 1
        
    for i in (+1,-1):
        for j in (-1, +1):
            d = pos[0] + i, pos[1] + j
            if d in astar.moveable and ((d[0], d[1] - j) not in astar.moveable or (d[0] - i, d[1]) not in astar.moveable) and d in l:
                l.remove(d)
    return l

def init(start, end, moveable):
    astar.start = start
    astar.end = end
    astar.moveable = moveable
    
    #Algorithm initialization
    astar.open = [start]
    astar.closed = []
    
    astar.g = {start: 0}
    astar.h = {}
    astar.f = {}
    astar.parent = {start: None}
    
    astar.f[start] = astar.h[start] = h_function(start,end)
    
    astar.lowest = start
    astar.cursor = start
    astar.path = None
    
    astar.running = True
    
    astar.maxg = 0
    astar.maxh = 0
    
    astar.current = None
    
    astar.checked_neighbors = []
    astar.checked_nodes = []
def nodeCmp(x,y):
    h1, h2 = h[x], h[y]
    return 1 if h2 > h1 else (0 if h1 == h2 else -1)

def next():
    if len(astar.open) > 0:
        current = None
        for node in astar.open:
            if not current or astar.f[node] < astar.f[current]:
                current = node
        
        astar.current = current
        
        if astar.h[current] < astar.h[astar.lowest] and astar.g[current] > astar.g[astar.lowest]:
            astar.lowest = current
        
        if astar.h[current] < astar.h[astar.cursor] or astar.g[current] > astar.g[astar.cursor]:
            astar.cursor = current
                
        if astar.lowest == astar.end:
            _stop()
            return False
        
        astar.open.remove(current)
                
        astar.closed.append(current)
        
        astar.checked_nodes = [current]
        
        for neighbor in astar.get_neighbors(current):
            update = False
            
            if neighbor not in astar.closed:
                
                astar.checked_nodes.append(neighbor)
                
                g = astar.g[current] + g_function(current, neighbor)
                
                if neighbor in astar.open:
                    if g < astar.g[neighbor]:
                        update = True
                else:
                    astar.open.append(neighbor)
                    update = True
            
                if update:                        
                    astar.parent[neighbor] = current
                    
                    astar.g[neighbor] = g
                    h = astar.h[neighbor] = h_function(neighbor, astar.end)
                    astar.f[neighbor] = g + h
                    
                    astar.maxg = max(astar.maxg, float(g))
                    astar.maxh = max(astar.maxh , float(h))

        #astar.open.sort(nodeCmp,reverse = True)
        
        return True
    else:
        _stop()
        return False
    
def _stop():
    astar.running = False
    astar.path = []
    node = astar.lowest
    while node:
        astar.path.insert(0,node)
        node = astar.parent[node]
            
def findpath(start, end, moveable):
    if astar.running:
      pass
    init(start,end,moveable)
    while next():
        pass
    return astar.path