"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import events
from animations import ResourceAnimation

class Buff(object):
	def __init__(self, attribute, modification, duration, cooldown=0, onlyOnce=False):
		object.__init__(self)
		self.attribute = attribute
		self.name = self.__class__.__name__
		self.modification = modification
		self.duration = duration
		self.cooldown = cooldown
		
		self.onlyOnce = onlyOnce
		self.applied = False
		
		self.curCooldown = 0
		
		self.active = True
	
	@property
	def applicable(self):
		return self.curCooldown <= 0

	@property
	def dead(self):
		if self.onlyOnce:
			return self.applied
		
		return self.duration < 0
	
	def update(self, tick):
		self.duration -= tick
		self.curCooldown -= tick
	
	def _modificate_attribute(self, attributeValue):
		raise NotImplementedError("%s needs to implement _modificate_attribute." % self.__class__.__name__)
	
	def apply(self, attributeValue):
		if not self.applicable:
			return attributeValue
		
		self.curCooldown = self.cooldown
		self.applied = True
		return self._modificate_attribute(attributeValue)		

class HoneyTowerSpeedBuff(Buff):
	def __init__(self, **kwargs):
		Buff.__init__(self, "speed", kwargs["modification"], kwargs["duration"])
	
	def _modificate_attribute(self, attributeValue):
		return attributeValue / self.modification

class HoneyTowerHPBuff(Buff):
	def __init__(self, **kwargs):
		Buff.__init__(self, "hp", kwargs["modification"], kwargs["duration"], onlyOnce=True)
	
	def _modificate_attribute(self, attributeValue):
		return attributeValue * self.modification

class InsecticideTowerHPBuff(Buff):
	def __init__(self, **kwargs):
		Buff.__init__(self, "hp", kwargs["modification"], kwargs["duration"], kwargs["cooldown"])
		
		self.pos = (0,0)
	
	def set_position(self, pos):
		self.pos = pos
	
	def _modificate_attribute(self, attributeValue):
		events.add_animation(ResourceAnimation(self.pos, self.modification, (255,0,0,200)))
		return attributeValue + self.modification