"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame
import cfg

class Drawable(pygame.sprite.DirtySprite):
#class Drawable(pygame.sprite.Sprite):
	"""Baseclass for all Drawable objects"""
	visreg_topleft = (0,0)
	visreg_bottomright = (1,1)
	
	def __init__(self):
		pygame.sprite.Sprite.__init__(self)
		self.rect = ((0,0),(1,1))
	
	def click_check(self, pos):
		return self.rect.collidepoint((pos[0], pos[1]))
	
	@classmethod
	def set_visible_region(cls, topleft=None, bottomright=None):
		cls.visreg_topleft = topleft
		cls.visreg_bottomright = bottomright
	
	def update(self, tick):
		raise NotImplementedError("Drawable needs to implement update!")
	
	@property
	def visible(self):
		if self.rect.topleft[0] > Drawable.visreg_bottomright[0]\
		or self.rect.bottomright[0] < Drawable.visreg_topleft[0]\
		or self.rect.bottomright[1] < Drawable.visreg_topleft[1]\
		or self.rect.topleft[1] > Drawable.visreg_bottomright[1]:
			self.dirty = 2
			return 0
		
		self.dirty = 1
		return 1

class Animation(Drawable):
	"""Baseclass for all animations."""
	
	def __init__(self, center=(0,0), frames=[], duration=0, loop=False, size=None):
		if not (frames or size):
			return None
		
		Drawable.__init__(self)
		
		self.frame = 0
		self.lastframetime = 0
		self.frames = frames
		self.lastframetime = 0
		self.duration = duration
		if self.frames:
			self.framedelay = duration / len(self.frames)
		else:
			self.framedelay = 100000
		
		if not size and self.frames:
			size = self.frames[0].get_rect().size
		
		self.surface = pygame.Surface(size, cfg.SURFACEFLAGS)
		self.rect = self.surface.get_rect()
		self.rect.center = center
		
		self.loop = loop
	
	def reset_delay(self):
		self.framedelay = self.duration / len(self.frames)
	
	@classmethod
	def get_animation_images(cls, image, framewidth):
		frames = []
		
		height = image.get_rect().h
		for offset in xrange(0, image.get_rect().w, framewidth):
			surface = pygame.Surface((framewidth, height), cfg.SURFACEFLAGS)
			surface.fill((0,0,0,0))
			area = pygame.Rect(offset, 0, offset + framewidth, height)
			surface.blit(image, (0,0), area)
			frames.append(surface)
		
		return frames

	@property
	def image(self):
		return self.surface
	
	@property
	def finished(self):
		return self.frame >= len(self.frames) and not self.loop
	
	def frame_switch(self, force=False):
		"""Switches the frames of the animation, if necessary.
		
		self.lastframetime has to be set before calling this function.
		"""
		# return if the frame should not be switched
		if self.lastframetime < self.framedelay and not force:
			return
		
		# calculate the frame that should be shown
		frameoffset = self.lastframetime / self.framedelay
		self.frame += frameoffset
		
		if self.finished:
			return
		else:
			self.frame %= len(self.frames)
		
		self.surface.fill((0,0,0,0))
		self.surface.blit(self.frames[self.frame], (0,0))
		
		# recalculate the frame time
		self.lastframetime -= frameoffset * self.framedelay