"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, astar, time
from vector2D import Vector2D
from random import uniform
from tile import Tile
import cfg

from drawables import Animation

from enemycfg import enemycfgs

class Enemy(Animation):
	"""Base class for all enemies."""
	
	def __init__(self, controller, identifier, waveNr, hp=-1, speed=-1, res=-1): #controller, name, death_sound, image_w, image_tr, image_tl=None):
		"""Initializes the enemy.
		
		@param controller Enemycontroller.
		@param identifier The name of the enemy.
		@param hp The healthpoints of the enemy.
		@param speed The speed of the enemy.
		@param death_sound Sound of the bug dying.
		"""
		frames_tr = Animation.get_animation_images(enemycfgs[identifier]["image_tr"], enemycfgs[identifier]["image_w"])
		if enemycfgs[identifier]["image_tl"]:
			frames_tl = self.get_animation_images(enemycfgs[identifier]["image_tl"], enemycfgs[identifier]["image_w"])
		else:
			frames_tl = rames_tr
		
		Animation.__init__(self, (0,0), frames_tr, 10, True)
		
		self.controller = controller
		self.name = identifier
		self.waveNr = waveNr
		self.maxSpeed = speed
		self.speed = speed
		self.hp = hp
		self.maxHp = hp
		self.res = res
		self.partTile = 10
		self.frame = 0
		self.framedelay = 0
		self.animSpeed = 400
		self.frames_tr = frames_tr
		self.frames_tl = frames_tl
		self.config = enemycfgs[self.name]
		self.hover = False
		self.buffs = []
		self.displayname = self.config["name"]
		
		if self.hp == -1:
			self.hp = self.config["hp"]
			self.maxHp = self.hp
		if self.speed == -1:
			self.maxSpeed = self.config["speed"]
			self.speed = self.maxSpeed
		if self.res == -1:
			self.res = self.config["res"]
			
		self.movement = self.config["movement"]
		self.death_sound = self.config["death_sound"]
		
		self.resistance = self.config["resistance"]
		
		self.movementdirection = Vector2D()
		
		self.running = True		
		self.surface = pygame.Surface((self.frames_tl[0].get_rect().w, self.frames_tl[0].get_rect().h), cfg.SURFACEFLAGS)
		self.rect = self.surface.get_rect()
		
		self.position = Vector2D(self.rect.center)
		self.path = None
		self.pathNode = None
		self.tileNr = 0
		self.direction = 1
		
		self.tilex = Tile.vecsize.x/2
		self.tilex += int(uniform(-self.tilex, self.tilex))
		self.tiley = Tile.vecsize.y/2
		self.tiley += int(uniform(-self.tiley, self.tiley))
	
	@property
	def image(self):
		return self.surface
	
	def animate(self, tick):
		self.framedelay += tick
		if self.framedelay > self.animSpeed:
			nextframe = self.framedelay / self.animSpeed
			self.surface.fill((0,0,0,0))
			
			if self.movementdirection[0] < 0:
				self.frame = (self.frame + nextframe) % len(self.frames_tl)
				self.surface.blit(self.frames_tl[self.frame], (0,0))
			else:
				self.frame = (self.frame + nextframe) % len(self.frames_tr)
				self.surface.blit(self.frames_tr[self.frame], (0,0))
			
			self.framedelay -= nextframe * self.animSpeed
	
	def set_position(self, pos):
		self.position = pos
		rectx = pos[0] + self.tilex
		recty = pos[1] + self.tiley
		self.rect.center = (rectx, recty)
		
	
	def get_speed(self, tick):
		"""Returns the speed for the current frame.
		
		@param tick Time since the last screen refresh.
		"""
		return max(1,int(uniform(self.maxSpeed/2, self.maxSpeed))*tick/1000)
		#return self.maxSpeed * tick / 1000
	
	def _add_buff(self, buff):
		oldbuffs = filter(lambda invalidBuff: invalidBuff.name == buff.name, self.buffs)
		for oldbuff in oldbuffs:
			self.buffs.remove(oldbuff)
		
		self.buffs.append(buff)
	
	def add_buffs(self, buffs):
		if isinstance(buffs, list):
			for buff in buffs:
				self._add_buff(buff)
		else:
			self._add_buff(buffs)
	
	def apply_buffs(self, tick, attribute):
		self.buffs = filter(lambda buff: not buff.dead, self.buffs)
		
		for buff in filter(lambda buff: buff.attribute == attribute, self.buffs):
			buff.update(tick)
			
			if not buff.applicable and not buff.dead:
				continue
			
			if hasattr(self, attribute):
				setattr(self, attribute, buff.apply(getattr(self, attribute)))
	
	def set_buff_position(self, pos):
		for buff in filter(lambda buff: hasattr(buff, "set_position"), self.buffs):
			buff.set_position(pos)

	def update(self, tick):
		self.animate(tick)
		
		if not self.path:
			if self.movement == "air":
				self.path = self.controller.pathfinder.get_airpath(self.way)
			else: 	
				self.path = self.controller.pathfinder.get_path(self.way)
			return
		if self.path:
			if not self.pathNode:
				pathNode = self.controller.level.tile_to_pxl(Vector2D(self.path.get_start_pos()))			
			
			if self.position == self.pathNode:
				self.tileNr += self.direction
				pathNode = self.get_pathTile()
				
				if self.pathNode == pathNode:
					self.running = False
					# self.direction = -self.direction
					# pathNode = self.get_pathTile()
				
				self.pathNode = pathNode
				
				self.speed = 1000 / self.get_speed(tick) + 1
				self.apply_buffs(tick, "speed")
				self.speed = int(self.speed)
			
			self.movementdirection = self.pathNode - self.position
			self.movementdirection.div(self.speed)
			self.position.add(self.movementdirection)
			self.set_position(self.position)
			if self.speed >= 1:
				self.speed -= 1
		
		self.set_buff_position(self.rect.center)
		self.apply_buffs(tick, "hp")
	
	def get_pathTile(self):
		pathNode = None
		if self.direction == 1:
			pathNode = self.controller.level.tile_to_pxl(self.path.get_next_tile(self.tileNr)[1])
		else:
			pathNode = self.controller.level.tile_to_pxl(self.path.get_previous_tile(self.tileNr)[1])
		return pathNode
	
	def hasarrived(self):
		return not self.running
	
	def get_resistance_to(self, damage_type):
		"""Returns the resistance to the given damage type.
		
		This method considers default values.
		
		@param damage_type the type of damage to which the resistance is wanted
		@return the damage that the bug is resistant against."""
		
		if damage_type in self.resistance.keys():
			return self.resistance[ damage_type ]
		
		return 0
