"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame
import Queue

__eventQueue = Queue.Queue()

def __addEvent(event):
	__eventQueue.put(event)

def get_events():
	res = []
	while not __eventQueue.empty():
		res.append(__eventQueue.get())
	return res

def add_build_tower(weaponInstance):
	"""Create an event to place a tower on the map.
	
	@param weaponInstance instance of Weapon that is supposed to be built.
	"""
	__addEvent(__createUserEvent("BuildTower", weapon=weaponInstance))

def add_resources(resources):
	"""Adds resources to the players account."""
	__addEvent(__createUserEvent("AddResources", resources=resources))

def add_animation(animation):
	"""Adds a weapon animation to the playground."""
	__addEvent(__createUserEvent("Animation", animation=animation))

def add_detected_gesture(gesture):
	"""Adds a detected gesture to the event queue."""
	__addEvent(__createUserEvent("Gesture", gesture=gesture))

def __createUserEvent(name, **kwargs):
	"""Create an instance.
	
	@param name name the identifier of the event
	@param kwargs the user-defined event params
	"""
	kwargs["name"] = name
	
	#return pygame.event.Event(pygame.USEREVENT, kwargs)
	return kwargs