"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame
import os
import glob

from resourcemanager import ResourceManager
from menuitems import MenuItem
from game import Game
from tower import TowerMenu
from level import Level, LevelFactory

class _LevelMenuItem(MenuItem):
	"""A menu entry representing a game level."""
	
	def __init__(self, level, enabled=False):
		"""
		@param itemdict {value -> item}
		"""
		MenuItem.__init__(self, "level.png", "level_off.png", enabled)
		
		self.level = level

		# Render the size
		font = ResourceManager.get_font(20)
		x_img = font.render(str(int(level.size.x)), True, (0,0,0,255))
		y_img = font.render(str(int(level.size.y)), True, (0,0,0,255))
		
		# Draw the size
		size_x = 5
		size_y = self.imageActive.get_rect().height / 2 - max(x_img.get_rect().height,y_img.get_rect().height) / 2
		self.imageActive.blit(x_img, (size_x, size_y))
		self.imageActive.blit(y_img, (size_x + 40, size_y))
		self.imageInactive.blit(x_img, (size_x, size_y))
		self.imageInactive.blit(y_img, (size_x + 40, size_y))
		
		# Render the name.
		font = ResourceManager.get_font(30)
		name_img = font.render(level.name, True, (0,0,0,255))
		
		# Draw the name.
		name_x = 100
		name_y = self.imageActive.get_rect().height / 2 - name_img.get_rect().height / 2
		self.imageActive.blit(name_img, (name_x,name_y))
		self.imageInactive.blit(name_img, (name_x,name_y))
		
		# Load the level image.
		self.preview_image = self.level.image  # ResourceManager.get_image(self.level.name + "_.jpg")
		
		# Scale the image.
		max_size = 200
		rect = self.preview_image.get_rect()
		if rect.width > rect.height:
			width = max_size
			height = int(rect.height * (float(width) / rect.width))
		else:
			height = max_size
			width = int(rect.width * (float(height) / rect.height))
		self.preview_image = pygame.transform.scale(self.preview_image, (width, height))
		

from menu import Menu
class LevelMenu(Menu):
	def __init__(self, screen, runner, parent, endlessmode=False):
		Menu.__init__(self, screen, runner, parent)
		self.endlessmode = endlessmode

	def start_level(self):
		selected_level = self.menuItems[ self.currentItem ].level
		g = Game(selected_level, TowerMenu(self.screen), self.screen, self.endlessmode)
		g.run()
		g.stop()
	
	def _determine_existing_levels(self):
		levels = []
		
		# Iterate over all level files.
		pattern = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "data", "maps", "*.lvl")
		for infile in glob.glob( pattern ):
			levelname = infile[:len(infile)-4] # Remove the file extension
			levelname = levelname[levelname.rfind(os.sep)+1:]
			try:
				levels.append(LevelFactory.get_Level(levelname))
			except Exception, e:
				print "Could not load level " + levelname + "!\n%s" % e
			
		return levels
	
	def initiate_menu_items(self):
		levels = self._determine_existing_levels()
		
		for level in levels:
			item = _LevelMenuItem(level, True)
			item.add_action(self.start_level)
			self.add_menu_item(item)
		
		item = MenuItem("back.png", "back_off.png", True)
		item.add_action(self.back_to_parent)
		self.add_menu_item(item)


	def draw(self):
		# Draw the background to get rid of the preview image.
		self.draw_background()
		
		# Draw the level image of the currently selected item.
		currentItem = self.menuItems[ self.currentItem ]
		if isinstance(currentItem, _LevelMenuItem):
			y = currentItem.position[1]
			self.screen.blit(currentItem.preview_image, (20, y), (0,0,200,200))
			