"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame

import cfg

from drawables import Drawable
from vector2D import Vector2D

class Radar(Drawable):
	
	def __init__(self, level, enemycontroller):
		Drawable.__init__(self)
		
		self.level = level
		self.enemycontroller = enemycontroller
		
		# What dimensions does the radar have?
		max_size = Vector2D(100,100)
		
		# Get an accordingly scaled copy of the radar image.
		img = level.preview
		rect = img.get_rect()
		if rect.width > rect.height:
			width = int(max_size.x)
			height = int(rect.height * (float(width) / rect.width))
		else:
			height = int(max_size.y)
			width = int(rect.width * (float(height) / rect.height))
		self.radar_image = pygame.transform.scale(img, (width, height))
		self.img_size = Vector2D(width,height)

		# Make the surface as large.		
		self.size = Vector2D(width,height)
		self.surface = pygame.Surface(self.size.to_tuple(), cfg.SURFACEFLAGS)

		# Set the background-colors for the window, a transparent grey
		self.bg_color = (120,120,120,120)

	@property
	def image(self):
		return self.surface
	
	def update(self, tick):
		pass
	
	def draw(self):
		# Fill the screen with its background color.
		self.surface.fill(self.bg_color)
		
		# Draw the radar image.
		self.surface.blit(self.radar_image, (0,0))
		
		# Draw the enemies.
		for enemy in self.enemycontroller.enemyGroup:
			pos = self.level_to_radar(enemy.position)
			pygame.draw.circle(self.surface, (255,0,0,255), pos, 1)
	
	def level_to_radar(self, pos):
		x = float(pos.x) / self.level.pxl_width * self.img_size.x
		y = float(pos.y) / self.level.pxl_height  * self.img_size.y
		return (int(x), int(y))

