"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame, os
pygame.init()

class _NoneSound(object):
	"""A NoneSound Wrapper Class.
	
	This is an empty wrapper. If the sound system could not be initialized,
	this class will be returned and simply does nothing.
	"""
	
	def play(self):
		"""Plays the NoneSound."""
	
	def stop(self):
		"""Stops playing the NoneSound"""

class _ResourceManager(object):
	def __init__(self):
		self.images = {}
		self.sounds = {}
		self.fonts = {}
		self.levels = {}
		
		#self.dirname = os.path.dirname(os.path.realpath(__file__) + os.sep + "..")
		self.dirname = os.path.dirname(os.path.realpath(__file__) )
		
		self.respath = os.path.join(self.dirname, "..", "data")
		
		self.imagepath = os.path.join(self.respath, "images")
		self.soundpath = os.path.join(self.respath, "sounds")
		self.levelpath = os.path.join(self.respath, "maps")		
		self.fontpath = os.path.join(self.respath, "font", "Carolingia.ttf")
	
	def _load_image(self, filepath):
		try:
			image = pygame.image.load(filepath)
		except pygame.error:
			raise SystemError("Cannot load image: %s" % filepath)
		
		return image

	def _load_sound(self, filepath):
		if not pygame.mixer or filepath == None:
			return _NoneSound()
		
		try:
			sound = pygame.mixer.Sound(filepath)
		except pygame.error, e:
			raise SystemError("Cannot load sound: %s." % filepath)
		
		return sound

	def get_image(self, file=None):
		if not file in self.images:
			image = self._load_image(os.path.join(self.imagepath, file))
			self.images[file] = image
		
		return self.images[file].copy()

	def get_sound(self, file=None):
		if not file in self.sounds:
			sound = self._load_sound(os.path.join(self.soundpath, file))
			sound.set_volume(.2)
			self.sounds[file] = sound
		
		return self.sounds[file]
	
	def get_font(self, fontsize):
		if not fontsize in self.fonts:
			self.fonts[fontsize] = pygame.font.Font(self.fontpath, fontsize)
		
		return self.fonts[fontsize]
	
	def get_level_path(self, level):
		if not level in self.levels:
			self.levels[level] = os.path.join(self.levelpath,  level + ".lvl")
		
		return self.levels[level]

ResourceManager = _ResourceManager()