"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import os
from resourcemanager import ResourceManager

__imgbase = "towers" + os.sep
__soundbase = "towers" + os.sep

weaponcfgs = {
	"InsecticideTower" :[{
							"name":      "Insecticide Tower",
							"shortname": "Insecticide/1",
							"orgimage":  ResourceManager.get_image(__imgbase + "insecticidetower_l1.png"),
							"size":      (21, 50),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "insecticidetower_icon.png"),
							"sound":      ResourceManager.get_sound(__soundbase + "insecticidetower_attack.wav"),
							"attAnimImg":__imgbase + "insecticidetower_attack.png",
							"costs":     120,
							"cooldown":  4000,
							"range":     5,
							"damage":    30,
							# Each weapon must specify exactly one damage type.
							# The type affects the damage that the tower inflicts on a certain bug,
							# since bugs have different resistance to different types of damage.
							# Possible types are: chemical, impact, biological
							"damagetype": "chemical",
							"attackables": ["air", "earth"],
							"buffs":       [{"name": "InsecticideTowerHPBuff", "duration": 5000, "modification": -5, "cooldown": 1000}],
							# Area of Effect, False for single-enemy-damage, number x
							# for damage to all enemies on a quadrat with edge length of x tiles
							"aoe":		 2,
						},
						{
							"name":      "Insecticide Tower",
							"shortname": "Insecticide/2",
							"orgimage":  ResourceManager.get_image(__imgbase + "insecticidetower_l2.png"),
							"size":      (21, 50),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "insecticidetower_icon.png"),
							"sound":      ResourceManager.get_sound(__soundbase + "insecticidetower_attack.wav"),
							"attAnimImg":__imgbase + "insecticidetower_attack.png",
							"costs":     150,
							"cooldown":  4000,
							"range":     5,
							"damage":    70,
							# Each weapon must specify exactly one damage type.
							# The type affects the damage that the tower inflicts on a certain bug,
							# since bugs have different resistance to different types of damage.
							# Possible types are: chemical, impact, biological
							"damagetype": "chemical",
							"attackables": ["air", "earth"],
							"buffs":       [{"name": "InsecticideTowerHPBuff", "duration": 6000, "modification": -10, "cooldown": 1000}],
							# Area of Effect, False for single-enemy-damage, number x
							# for damage to all enemies on a quadrat with edge length of x tiles
							"aoe":		 2,
						},
						{
							"name":      "Insecticide Tower",
							"shortname": "Insecticide/3",
							"orgimage":  ResourceManager.get_image(__imgbase + "insecticidetower_l3.png"),
							"size":      (21, 50),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "insecticidetower_icon.png"),
							"sound":      ResourceManager.get_sound(__soundbase + "insecticidetower_attack.wav"),
							"attAnimImg":__imgbase + "insecticidetower_attack.png",
							"costs":     180,
							"cooldown":  4000,
							"range":     5,
							"damage":    100,
							# Each weapon must specify exactly one damage type.
							# The type affects the damage that the tower inflicts on a certain bug,
							# since bugs have different resistance to different types of damage.
							# Possible types are: chemical, impact, biological
							"damagetype": "chemical",
							"attackables": ["air", "earth"],
							"buffs":       [{"name": "InsecticideTowerHPBuff", "duration": 7000, "modification": -15, "cooldown": 1000}],
							# Area of Effect, False for single-enemy-damage, number x
							# for damage to all enemies on a quadrat with edge length of x tiles
							"aoe":		 2,
						}],
	"FlyFlapTower":		[{
							"name":      "Fly Flap Tower",
							"shortname": "FlyFlap/1",
							"orgimage":  ResourceManager.get_image(__imgbase + "flyflap_l1.png"),
							"size":      (33, 49),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "flyflap_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "flyflap_attack.wav"),
							"attAnimImg":__imgbase + "flyflap_attack.png",
							"costs":     80,
							"cooldown":  550,
							"range":     3,
							"damage":    20,
							"damagetype": "impact",
							"attackables": ["earth"],
							"aoe":		 1
						},
						{
							"name":      "Fly Flap Tower",
							"shortname": "FlyFlap/2",
							"orgimage":  ResourceManager.get_image(__imgbase + "flyflap_l2.png"),
							"size":      (33, 49),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "flyflap_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "flyflap_attack.wav"),
							"attAnimImg":__imgbase + "flyflap_attack.png",
							"costs":     80,
							"cooldown":  600,
							"range":     3,
							"damage":    40,
							"damagetype": "impact",
							"attackables": ["earth"],
							"aoe":		 1
						},
						{
							"name":      "Fly Flap Tower",
							"shortname": "FlyFlap/3",
							"orgimage":  ResourceManager.get_image(__imgbase + "flyflap_l3.png"),
							"size":      (33, 49),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "flyflap_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "flyflap_attack.wav"),
							"attAnimImg":__imgbase + "flyflap_attack.png",
							"costs":     120,
							"cooldown":  450,
							"range":     4,
							"damage":    55,
							"damagetype": "impact",
							"attackables": ["earth"],
							"aoe":		 1
						}],
	"FrogTower":    	[{
							"name":      "Toad Tower",
							"shortname": "Toad/1",
							"orgimage":  ResourceManager.get_image(__imgbase + "frogtower_l1.png"),
							"size":      (29, 28),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "frogtower_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "frogtower_attack.wav"),
							#"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     50,
							"cooldown":  1800,
							"range":     4,
							"damage":    145,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"aoe":		 False
						},
						{
							"name":      "Toad Tower",
							"shortname": "Toad/2",
							"orgimage":  ResourceManager.get_image(__imgbase + "frogtower_l2.png"),
							"size":      (29, 28),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "frogtower_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "frogtower_attack.wav"),
							#"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     50,
							"cooldown":  2000,
							"range":     4,
							"damage":    300,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"aoe":		 False
						},
						{
							"name":      "Toad Tower",
							"shortname": "Toad/3",
							"orgimage":  ResourceManager.get_image(__imgbase + "frogtower_l3.png"),
							"size":      (29, 28),
							"animDuration": 100000,
							"icon":      ResourceManager.get_image(__imgbase + "frogtower_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "frogtower_attack.wav"),
							#"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     50,
							"cooldown":  2000,
							"range":     4,
							"damage":    450,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"aoe":		 False
						}],
	"HoneyPotTower":   	[{
							"name":      "Honey Pot Tower",
							"shortname": "HoneyPot/1",
							"orgimage":  ResourceManager.get_image(__imgbase + "honeypot_l1.png"),
							"size":      (45, 40),
							"animDuration": 2000,
							"icon":      ResourceManager.get_image(__imgbase + "honeypot_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "honeypot_attack.wav"),
							"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     150,
							"cooldown":  2000,
							"range":     3,
							"damage":    5,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"buffs":     [{"name": "HoneyTowerSpeedBuff", "duration": 5000, "modification": 0.6}, {"name": "HoneyTowerHPBuff", "duration": 0, "modification": 1.2}],
							"aoe":		 1
						},
						{
							"name":      "Honey Pot Tower",
							"shortname": "HoneyPot/2",
							"orgimage":  ResourceManager.get_image(__imgbase + "honeypot_l2.png"),
							"size":      (45, 40),
							"animDuration": 2000,
							"icon":      ResourceManager.get_image(__imgbase + "honeypot_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "honeypot_attack.wav"),
							"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     50,
							"cooldown":  3000,
							"range":     4,
							"damage":    10,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"buffs":     [{"name": "HoneyTowerSpeedBuff", "duration": 6000, "modification": 0.5}, {"name": "HoneyTowerHPBuff", "duration": 0, "modification": 1.1}],
							"aoe":		 1
						},
						{
							"name":      "Honey Pot Tower",
							"shortname": "HoneyPot/3",
							"orgimage":  ResourceManager.get_image(__imgbase + "honeypot_l3.png"),
							"size":      (45, 40),
							"animDuration": 2000,
							"icon":      ResourceManager.get_image(__imgbase + "honeypot_icon.png"),
							"sound":     ResourceManager.get_sound(__soundbase + "honeypot_attack.wav"),
							"attAnimImg":__imgbase + "honeypot_attack.png",
							"costs":     70,
							"cooldown":  3500,
							"range":     4,
							"damage":    20,
							"damagetype": "biological",
							"attackables": ["air", "earth"],
							"buffs":     [{"name": "HoneyTowerSpeedBuff", "duration": 7000, "modification": 0.4}],
							"aoe":		 2
						}]
}