"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from resourcemanager import ResourceManager

import pygame

class WindowContent(object):
	quitApp = False
	
	def __init__(self, screen, bgImage, runner, parent=None, useSkin=False):
		object.__init__(self)
		self.screen = screen
		self.bgImage = bgImage
		self.runner = runner
		self.parent = parent
		self.clock = pygame.time.Clock()
	
	def tick(self):
		"""Uses the PyGame clock to control the fps."""
		return self.clock.tick()
	
	def back_to_parent(self):
		if self.runner:
			self.shouldRun = False
			self.runner.runningContent = self.parent
	
	def enter_child(self, child):
		if self.runner and child:
			self.shouldRun = False
			self.runner.runningContent = child
	
	def draw_background(self):
		self.screen.blit(self.bgImage, (0,0))
	
	def handle_events(self):
		return True
	
	def execution_loop(self):
		pass
	
	def run(self):
		self.shouldRun = True
		self.draw_background()
		
		while self.handle_events() and not WindowContent.quitApp and self.shouldRun:
			self.execution_loop(self.tick())
			pygame.display.flip()
		
		if self.runner:
			if WindowContent.quitApp:
				self.runner.runningContent = None
			elif self.parent:
				self.back_to_parent()